;Trumpet Timer for Brian Lamb -- April 24, 2010
;
;
;  1). Push the button once to turn unit on. (OFF LOOP:  light is initially off - turned on when button is pushed)

;  2). Push the button again and it starts counting up.  (COUNTUP LOOP:  light comes on and stays on until second
;             push of button)

;  3). Push the button again and it counts back down from where you stopped. (COUNTDOWN LOOP: Light goes out
;             indicating countdown has started)

;        A push of button while running this Countdown Loop will go to step 1 (off loop)


;  4). Light comes on telling you that you came back to zero. Light stays on until you push button again , and it starts at #2 again.

;  5). Press TWO times to turn unit off. Light blinks twice.


;~~~~~~~~~~~~~~~~~~~~~~
;Trumpet Timer 4/19/2010 OLD Trumper Timer INSTRUCTIONS
;
;(1)  (OFF LOOP) (COUNT UP LOOP = 1a) Push button once to start COUNT UP 
;	Light comes on and stays on until COUNT UP is stopped by second push
;   of the button.  Step 2 immediately follows.
;

;(2) COUNT DOWN immediately starts for the same amount of time determined by the 
;    COUNT UP.  Light goes off..
;
;	Light remains off until COUNT DOWN is complete at which time, three one second flashes
;   will be seen.  At end of this COUNT DOWN, it moves back to step (1).

;	If at any time during this COUNT DOWN cycle the button is pushed, the COUNT DOWN
;	is aborted.  The light flashes three times to indicate the interruption and stop.
;   After flashes, unit goes to step (1) above --- to await the next new COUNT UP..


;NOTES:  The power strip toggle switch must be ON in order to power up the micro-controller.  When 
;	this is done, you will see four flashes of the light --- indicating it is ready for use.

;	MAXIMUM power load capability is 150 watts at one outlet.  If more than one outlet is
;   used, total wattage must not exceed 150 watts.
;
;	Total power while the strip is ON (with the light off) is a little over 1/3 watt -- very green.
;   If left on all year, 24 hours a day, 365 days a year, it would use about 35-cents worth of
;   power at a cost of 10 cents/KWH.  No power is consumed when the power strip toggle
;   switch is turned off.
;
;	DO NOT try to add an external push button switch.  This micro-controller circuit is
;   connected directly to the AC line thus, the wires to an external switch would be
;   dangerously "hot."  

;~~~~~~~~~~~~~~~~~~~~







;Internal oscillator of ATTINY2313 is running micro-controller at 1MHz (Divide by 8 in fuses selected)


;April 13, 2010

;R18 -- Global Flags Register


;r18,0(1) = Light on (1)  -- Light off (0)
;r18,1(2) = 1/50th second flag (generated by interrupt every 50th of a second)
;r18,2(4) = 1/1000th second flag generated by interrupt
;r18,3(8) = Countdown complete flag
;r18,4(16) = End of flashing of light routine flag
;r18,5(32) =
;r18,6(64) =
;r18,7(128) = 

;r0 = global 20 counter to generate 1/50th second flag during interrupt


.include "tn2313def.inc"

rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt		;1,000 interruptions per second
rjmp	START1
rjmp	START1

	
START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,7		;Set bits 0-3 for output
out		DDRA,r16	;Execute data direction with PA0-3 as output

clr		r16
out		PortA,r16	;turn off ports

ldi		r16,126		;Set bits PD1-6 to ones for out, bit PD0 for input
out		DDRD,r16	;Execute data direction with PD0 as input rest as output

clr		r16
out		PortD,r16	;All output ports low
					;No input resistor for input port D,0

ser		R16			;Set all bits for data direction as outputs
out		DDRB,R16	;execute data direction with all as outputs
clr		r16
out		PORTB,R16	;execute to set out ports low

in		r16,TIMSK	;enable 16-bit match to A register
ori		r16,64		
out		TIMSK,r16

in		r16,TCCR1B
ori		r16,9		; 1=no prescaling, 2=/8 prescaling, 3=/64 prescaling, etc. (NO prescaling used here)
out		TCCR1B,r16	;enable reset of counter to zero (bit3(8)) after match detection.
					;and bits 0,1,2 select prescaling source as no prescaling of clock

ldi		r16,3
out		OCR1AH,r16
ldi		r16,232		;For 1/1,000th second interrupt frequency, H=3, L=232
out		OCR1AL,r16	;loads the compare register A for 1/1,000th second interrupt
					;interval (1,000,000 Hz clock frequency)	

clr		r18			;clear flags register

rcall	DelaySecond		;allow push button capacitor to charge

ldi		r16,20
sts		$61,r16				;Store count of 20 for 50th counter and store in SRAM at $61

bset	7					;enable global interrupts


sbic	PIND,0				;skip if PIND,0 is clear (switch is down)
rjmp	TTStart
rjmp	StartOff			; Hold switch down if one wishes to enter Old Trumpet Timer Code



;-----------------------------


StartOff:

rcall	Delay4Seconds
rcall	Delay4Seconds

ldi		r16,2
ldi		r20,25			;r16= number of flashes, r20 = length of flash
rcall	Flash			;flash light two times to indicate unit is "ready."


OffLoop:			;Step 1 (Switch must be up when entering this loop)

sbic	PIND,0		;skip if PIND,0 is clear (switch is down)
rjmp	OffLoop


OL1:				;Step 1a -- Count Up starts here

sbr		r18,1		;Set light on, r18,0(1)
cbr		r18,4		;clear r18,2(4), 1/1000th second flag
rcall	ClearTimeCounter


OL2:

rcall	IncrementCounter		;start counting up - 1/1000th second counter

sbis	PIND,0		;skip if PIND,0 is set (wait until switch is up)
rjmp	OL2


OL3:

rcall	IncrementCounter		;1/1000th second counter

sbic	PIND,0		;skip if PIND,0 is clear (wait until switch is down)
rjmp	OL3

cbr		r18,1				;clear r18,0(1)  LIGHT OFF


OL4:						;START OF STEP 2 - decrement counter until zero or interrupted

rcall	DecrementCounter

sbis	PIND,0				;skip if PIND,0 is set (switch is up)
rjmp	OL4


OL4a:

rcall	DecrementCounter

brcs	OL5					;Counter has reached ZERO
	
sbic	PIND,0				;skip if PIND,0 is clear (wait until switch is down)
rjmp	OL4a


OL4aa:

sbis	PIND,0				;skip if PIND,0 is set (switch is up)
rjmp	OL4aa					;count aborted - stop

rjmp	OL6					;abort countdown

;--------


OL5:				;Count is up

ldi		r16,3		;numbers of flashes = 3
ldi		r20,50		;a 1 second (50 count) flash is set

rcall	Flash

rjmp	OffLoop


OL6:				;count is aborted --- stop

ldi		r16,3		;three short flashes
ldi		r20,15		;set length of flash = 15 counts

rcall	Flash

rjmp	OffLoop


;-----------------------------

ClockCntDwnInt: 	;interrupt occurs every 1,000th second\		
					; Light is turned on if r18,0(1) flag is set

push	r16

in		r16,SREG	;save flags
sts		$60,r16

sbr		r18,4		;Set r18,2(4) bit = 1 1,1000 th Interrupt Occured Flag

sbrs	r18,0		;skip if r18,0(1) bit is set (1=Light On, 0= Light Off)
rjmp	CCDI1

sbi		PortB,1		;on PortB,1 for 5 usec pulse of triac

push	r17
pop		r17			;4 usec delay

cbi		PortB,1		;off PortB,1 - end of triac pulse


CCDI1:

lds		r16,$61		;get previous count for 1/50th second flag
dec		r16
brne	CCDI2

ldi		r16,20
sbr		r18,2		;set 1/50th second flag, r18,1(2)


CCDI2:

sts		$61,r16

lds		r16,$60		;Get SREG values
out		SREG,r16	;Restore SREG flag values

pop		r16

reti				;Return from Interrupt


;----------------------------------------

Delay01:			;Delay .01 seconds - 1/100 second

push	r17
ldi		r17,5


D01a:

rcall	Delay001803 	;rcall delay of .001803 seconds
dec 	r17
brne	D01a

pop		r17
ret

;----------------------------------------


Delay025:			; 1/40th second delay

push	r17
ldi		r17,13


D025a:

rcall	Delay001803	;rcall .001803 delay
dec		r17
brne	D025a

pop		r17
ret


;----------------------------------

DelaySecond:

push	r17
ldi		r17,40


DSa:

rcall	Delay025
dec		r17
brne	DSa

pop		r17
ret


;-------------------------------------

Delay4Seconds:

push	r17
ldi		r17,4

D4Sa:

rcall	DelaySecond
dec		r17
brne	D4Sa

pop		r17
ret


;----------------------------------

Delay001803:		;A .001803 second delay

push	r17
clr		r17


D001803a:

dec		r17
brne	D001803a

pop		r17
ret


;-------------------------------------

;Variable Delays

;RCALL (3)


Delay13:
nop
Delay12:
nop
Delay11:
nop
Delay10:
nop
Delay9:
nop
Delay8:
nop
Delay7:
ret					;(4) (5 total if this line called)


;---------------------------------

ClearTimeCounter:

clr		r26
clr		r27
clr		r28
clr		r29
clr		r30
clr		r31

ret

;---------------------------------

IncrementCounter:
;1,000 counts per second
;60,000 counts per minute
;3,600,000 counts per hour
;86,400,000 counts per day
;3.1536 X10 per year
;Counter is capable of 2.814749767 X14 counts .... or 8,925 years

;  This counter capable of 2.8 X 14 or 8,925 years of 1000 counts per second

cbr		r18,4		;clear r18,2(4), 1/1000th second flag 

adiw	r26,1		;add one to r26,27 word
brcc	ICend

adiw	r28,1		;add one to r28,29 word
brcc	ICend

adiw	r30,1		;add one to r30,31 word


ICend:

ret


;---------------------------------

DecrementCounter:	;1/1000th second --  carry flag is set when it reaches overflow (zero)
					
cbr		r18,12		;clear r18,2(4), 1,000th second flag
					;clear also, r18,3(8) - countdown complete flag

sbiw	r26,1		;subtract one from r26,27 pair
brcc	DCend

sbiw	r28,1		;subtract one from r28,29 pair
brcc	DCend

sbiw	r30,1
brcc	DCend		;carry will be set if overflow occurs

sbr		r18,8		;set r18,3(8), countdown complete flag


DCend:

ret


;-----------------------------------

Flash:				;Flash light variable number of times
					;r16 has number of flashes, r20 has length of flash (50 = one second)
					;gives number of flashes in r16 (minimum = 1)
					;Pause of 6 counts is placed between each flash

push	r22			;pause between flashes count
push	r23			;length of flash count

ldi		r22,18		;pause between flashes into r22
mov		r23,r20		;save r20 value in r23 -- length of flash


Flash1:

sbrs	r18,1		;skip if r18,1(2), 50th second flag is set
rjmp	Flash1


cbr		r18,2		;clear r18,1(2), 50th second flag

tst		r23			;test r23 (r20) for length of flash count
breq	Flash2

sbr		r18,1		;Set r18,0(1), light on
dec		r23			;decrement length of flash count

rjmp	Flash1		;go to Flash1


Flash2:				;Completed flash of light

cbr		r18,1		;off light, r18,0(1)

dec		r16			;decrement r16 - number of flashes
breq	Flash4

ldi		r22,6		;load pause between flashes value in r22 (6)


Flash3:	

sbrs	r18,1		;skip if r18,1(2) is set (50th flag)
rjmp	Flash3		;wait for 50th flag

cbr		r18,2		;clear 50th flag, r18,1(2)

dec 	r22			;decrement r22, pause count

brne	Flash3		;go through loop again

mov		r23,r20		;restore length of flash in r23
tst		r16
brne	Flash1


Flash4:				;out of loop

pop		r23
pop		r22

ret


;------------------------------------------

;METRONOME LOOP SECTION

;------------------------------------------


MetronomeLoopPre:

rcall	Delay4Seconds
rcall	Delay4Seconds


MetronomeLoopPre2:

ldi		r16,4
ldi		r20,7		;set up for Flash call - r16 has no. of flashes, r20=length of flash
rcall	Flash		;flash light four times to indicate "ready."


MetronomeLoop:

sbis	PIND,0		;skip if PIND,0 is set (switch is up)(switch is already down)
rjmp	MetronomeLoop


ML1:

sbic	PIND,0		;skip if PIND,0 is clear (switch is down)
rjmp	ML1


ML2:				;Start up count

sbr		r18,1		;set light on bit
rcall	ClearTimeCounter
cbr		r18,4		;clear r18,2(4), 1/1000th flag bit


ML2a:

sbrc	r18,2		;skip if r18,2(4) 1/1000th flag is clear
rcall	IncrementCounter	;1/1000th flag is reset by this call

sbis	PIND,0		;skip if PIND,0 is set (switch is up)
rjmp	ML2a		;jump back to repeat to wait for release of switch


ML2aa:

sbrc	r18,2		;skip if r18,2(4) 1/1000th flag is clear
rcall	IncrementCounter	;1/1000th flag is reset by this call

sbic	PIND,0		;skip if PIND,0 is clear (switch is down)
rjmp	ML2aa		;wait until switch is pressed


ML3:

cbr		r18,1		;Off light bit, r18,0(1)

mov		r3,r26		;capture delay time and store in registers r3-8
mov		r4,r27
mov		r5,r28
mov		r6,r29
mov		r7,r30
mov		r8,r31		;store present count into registers


ML3a:

sbis	PIND,0		;skip if PIND,0 is set (switch is up)
rjmp	ML3a		;wait until switch goes up


;START OF METRONOME FLASHING

MetronomeFlashout:

mov		r26,r3
mov		r27,r4
mov		r28,r5
mov		r29,r6
mov		r30,r7
mov		r31,r8		;restore metronome count


MF1:

sbrc	r18,2		;skip if r18,2(4) is clear - 1 1/1000th flag
rjmp	MF1b
rjmp	MF1a


MF1b:

rcall	DecrementCounter	;1/1000th r18,2(4) 1/1000th flag is reset by this call
sbrs	r18,3		;skip if r18,3(8) is set - down count complete flag
rjmp	MF1			;go for another round

cbr		r18,8		;clear r18,3(8) - down count complete flag
rcall	ToggleLight


MF1a:

sbis	PIND,0		;skip if PIND,0 is set - switch is up 
rjmp	MF2			;Escape

sbrs	r18,3		;skip if r18,3(8) is set (countdown complete flag)
rjmp	MetronomeFlashout	;go back





rcall	DecrementCounter
brcc	MF1			;branch back if carry clear


rcall	ToggleLight
rjmp	MetronomeFlashout


MF2:

cbr		r18,1		;Off light, clear r18,0(1)


MF2a:

sbis	PIND,0					;skip if PIND,0 is set  -- switch is up
rjmp	MF2a

rjmp	MetronomeLoopPre2		;jump and wait for new beat entry


;----------------------------------------

ToggleLight:		;Toggles light bit on or off

sbrs	r18,0		;skip if r18,0(1) set
rjmp	TL1

cbr		r18,1		;clear r18,0(1) 
ret


TL1:  

sbr		r18,1		;set r18,0(1)
ret


;-----------------------------------------

;TRUMPET TIMER SECTION -- 4/24/2010

;-----------------------------------------

; This section coded 4/24/2010 to be both a trumpet timer and metronone

; New 4/24/2010 Instructions:

; A single flash is seen after turn on of main power switch.

; First push of button starts the up counter
; A second push stops up counter.  The interval of time between these to flashes
;    is captured.
; The unit immediately goes into and on/off mode of the light.  The light comes on
;    for the same time as the interval programmed between the first two button pushes.
; The light goes off for the same interval programmed.
; This on/off process continues until a single button push stops the timing cycle.

; When interrupted, the unit returns to the beginning.  It awaits a new two button
;    button push for different timing cycle,

;------------------------------------------

TTStart:

rcall	Delay4Seconds
rcall	Delay4Seconds


TTStart2:

ldi		r16,1		;load one for a single light flash (r16=no. of flashes, r20=duration of flash)
ldi		r20,10		;load for a quick flash of the light - 10 cycles
rcall	Flash		;call for one quick light flash


TT1:

sbic	PIND,0		;skip if PIND,0 is clear, button is down
rjmp	TT1			;cycle back and wait for button push

rcall	ClearTimeCounter	;zero time count
sbr		r18,1				;turn on light


TT2:

sbic	PIND,0		;skip if PIND,0 is clear
rjmp	TT2a


TT2b:				;button is still pushed

sbrs	r18,2		;skip if r18,2(4) is set - 1/1000th flag
rjmp	TT2			;jump back if not set


rcall	IncrementCounter
rjmp	TT2			;jump back and see if switch is up yet


TT2a:				;button now up - continue

sbrs	r18,2		;skip if r18,2(4) is set - 1/1000th flag
rjmp	TT3


rcall	IncrementCounter


TT3:

sbic	PIND,0		;skip if button is pushed
rjmp	TT2a		;if button not pushed, recycle at TT2a


cbr		r18,1		;clear r18,0(1) bit - OFF light

mov		r3,r26		;capture delay time and store in registers r3-8
mov		r4,r27
mov		r5,r28
mov		r6,r29
mov		r7,r30
mov		r8,r31		;store present count into registers


TT4:

sbrs	r18,2		;skip if r18,2(4) is set - 1/1000th flag
rjmp	TT5

rcall	DecrementCounter	;subtract one from total count
sbrc	r18,3		;skip if r18,3(8) is clear - count is not zero yet
rjmp	TT6			;count is zero


TT5:

sbic	PIND,0		;skip in PIND,0 is clear
rjmp	TT7			;button finally up
rjmp	TT4			;got back until buttonis up


TT6:

mov		r26,r3
mov		r27,r4
mov		r28,r5
mov		r29,r6
mov		r30,r7
mov		r31,r8		;restore old count

rcall	ToggleLight	;change light state to opposite condition


TT7:

sbrs	r18,2		;skip if r18,2(4) is set - 1/1000th flag
rjmp	TT7b		;flag not set so got to TT7b


rcall	DecrementCounter
sbrc	r18,3		;skip if r18,3(8) is clear - count down complete flag
rjmp	TT6			;count is zero, start new count down using r3-r8 data


TT7b:

sbic	PIND,0		;skip if PIND,0 is clear 
rjmp	TT7			;switch is still up

cbr		r18,1		;button is pushed -- OFF light - clear r18,0(1)


TT8:				;button is down - stop process

sbis	PIND,0		;skip if PIND,0 is set
rjmp	TT8			;stall until button goes up

rcall	Delay025	;short delay
rjmp	TTStart2	;return to top and await new two button push timing cycle


;---------------------------------------










